"""Wrapper to get the measurings from a Luftdaten station."""
import logging

import httpx

from . import exceptions

_LOGGER = logging.getLogger(__name__)
_RESOURCE = "https://data.sensor.community/airrohr/v1"


class Luftdaten(object):
    """A class for handling the data retrieval."""

    def __init__(self, sensor_id):
        """Initialize the connection."""
        self.sensor_id = sensor_id
        self.data = None
        self.values = {}
        self.meta = {}
        self.url = "{}/{}".format(_RESOURCE, "sensor")

    async def get_data(self):
        """Retrieve the data."""
        url = "{}/{}/".format(self.url, self.sensor_id)

        try:
            async with httpx.AsyncClient() as client:
                response = await client.get(str(url))
        except httpx.ConnectError:
            raise exceptions.LuftdatenConnectionError(f"Connection to {url} failed")
        except httpx.ConnectTimeout:
            raise exceptions.LuftdatenConnectionError(f"Connection to {url} timed out")
        except httpx.ReadTimeout:
            raise exceptions.LuftdatenConnectionError(f"Read timeout from {url}")

        if response.status_code == httpx.codes.OK:
            try:
                _LOGGER.debug(response.json())
                self.data = response.json()
            except TypeError:
                _LOGGER.error("Can not load data from Luftdaten API")
                raise exceptions.LuftdatenConnectionError(
                    "Unable to get the data from Luftdaten API"
                )

        if not self.data:
            for measurement in self.values.keys():
                self.values[measurement] = None
            return

        try:
            sensor_data = sorted(
                self.data, key=lambda timestamp: timestamp["timestamp"], reverse=True
            )[0]

            for entry in sensor_data["sensordatavalues"]:
                if entry["value_type"] not in self.values.keys():
                    self.values[entry["value_type"]] = None
                for measurement in self.values.keys():
                    if measurement == entry["value_type"]:
                        self.values[measurement] = float(entry["value"])

            self.meta["sensor_id"] = self.sensor_id
            self.meta["longitude"] = float(sensor_data["location"]["longitude"])
            self.meta["latitude"] = float(sensor_data["location"]["latitude"])
            self.meta["altitude"] = float(sensor_data["location"]["altitude"])
        except (TypeError, IndexError):
            raise exceptions.LuftdatenError()

    async def validate_sensor(self):
        """Return True if the sensor ID is valid."""
        return True if self.data else False



"""Test the interaction with the Luftdaten API."""
import pytest
from pytest_httpx import HTTPXMock

from luftdaten import Luftdaten

SENSOR_ID = 1

RESPONSE_VALID = [
    {
        "sensor": {
            "sensor_type": {"manufacturer": "various", "name": "DHT22", "id": 9},
            "pin": "7",
            "id": 152,
        },
        "timestamp": "2021-11-07 11:16:45",
        "sampling_rate": None,
        "sensordatavalues": [
            {"value": "10.50", "id": 16932663249, "value_type": "temperature"},
            {"value": "79.30", "id": 16932663267, "value_type": "humidity"},
        ],
        "id": 7706863277,
        "location": {
            "country": "DE",
            "latitude": "48.792",
            "exact_location": 0,
            "altitude": "326.9",
            "longitude": "9.164",
            "indoor": 0,
            "id": 68,
        },
    },
    {
        "sensor": {
            "sensor_type": {"manufacturer": "various", "name": "DHT22", "id": 9},
            "pin": "7",
            "id": 152,
        },
        "timestamp": "2021-11-07 11:14:13",
        "sampling_rate": None,
        "sensordatavalues": [
            {"value": "10.50", "id": 16932614294, "value_type": "temperature"},
            {"value": "80.60", "id": 16932614337, "value_type": "humidity"},
        ],
        "id": 7706841422,
        "location": {
            "country": "DE",
            "latitude": "48.792",
            "exact_location": 0,
            "altitude": "326.9",
            "longitude": "9.164",
            "indoor": 0,
            "id": 68,
        },
    },
]


@pytest.mark.asyncio
async def test_sensor_values(httpx_mock: HTTPXMock):
    """Test the sensor values."""
    httpx_mock.add_response(json=RESPONSE_VALID)

    client = Luftdaten(SENSOR_ID)
    await client.get_data()

    assert client.values == {"temperature": 10.5, "humidity": 79.3}


@pytest.mark.asyncio
async def test_meta(httpx_mock: HTTPXMock):
    """Test the meta information."""
    httpx_mock.add_response(json=RESPONSE_VALID)

    client = Luftdaten(SENSOR_ID)
    await client.get_data()

    assert client.meta["sensor_id"] == 1
    assert client.meta["latitude"] == 48.792
    assert client.meta["longitude"] == 9.164
    assert client.meta["altitude"] == 326.9



"""Test the interaction with the Luftdaten API."""
import httpx
import pytest
from pytest_httpx import HTTPXMock

from luftdaten import Luftdaten
from luftdaten.exceptions import LuftdatenConnectionError

SENSOR_ID = 1


@pytest.mark.asyncio
async def test_connect_timeout(httpx_mock: HTTPXMock):
    """Test if the connection is hitting the timeout during connect."""

    def raise_timeout(request):
        """Set the timeout for the requests."""
        raise httpx.ConnectTimeout(
            f"Unable to connect within {request.extensions['timeout']}", request=request
        )

    httpx_mock.add_callback(raise_timeout)

    with pytest.raises(LuftdatenConnectionError):
        client = Luftdaten(SENSOR_ID)
        await client.get_data()


@pytest.mark.asyncio
async def test_read_timeout(httpx_mock: HTTPXMock):
    """Test if the connection is hitting the timeout during data reading."""

    def raise_timeout(request):
        """Set the timeout for the requests."""
        raise httpx.ReadTimeout(
            f"Unable to read within {request.extensions['timeout']}", request=request
        )

    httpx_mock.add_callback(raise_timeout)

    with pytest.raises(LuftdatenConnectionError):
        client = Luftdaten(SENSOR_ID)
        await client.get_data()



"""Example for getting the data from a station."""
import asyncio

from luftdaten import Luftdaten

SENSOR_ID = 152


async def main():
    """Sample code to retrieve the data."""
    data = Luftdaten(SENSOR_ID)
    await data.get_data()

    if not await data.validate_sensor():
        print("Station is not available:", data.sensor_id)
        return

    if data.values and data.meta:
        # Print the sensor values
        print("Sensor values:", data.values)

        # Print the coordinates fo the sensor
        print(
            "Location:",
            data.meta["latitude"],
            data.meta["longitude"],
            data.meta["altitude"],
        )


if __name__ == "__main__":
    asyncio.run(main())



"""Exceptions for the Luftdaten Wrapper."""


class LuftdatenError(Exception):
    """General LuftdatenError exception occurred."""

    pass


class LuftdatenConnectionError(LuftdatenError):
    """When a connection error is encountered."""

    pass


class LuftdatenNoDataAvailable(LuftdatenError):
    """When no data is available."""

    pass
